// Written by Ken Keefe
// Copyright 2005 by Ken Keefe
// This software is freely available for use by anyone.

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

char * HELP = "Usage: tcom SECONDS COMMAND\n\nWritten by Ken Keefe (c) 2005.\n";

pid_t cpid;

void shutitdown(int signo){
  if(SIGUSR1 == signo){
    exit(0);
  }
  if(SIGUSR2 == signo){
    kill(cpid, SIGTERM);
    exit(1);
  }
}

int main(int argc, char *argv[])
{
  if(argc < 3){
    printf(HELP);
    exit(1);
  }

  struct sigaction act;
  act.sa_handler = shutitdown;
  act.sa_flags = 0;
  if((sigemptyset(&act.sa_mask) == -1) || (sigaction(SIGUSR1, &act, NULL) == -1) || (sigaction(SIGUSR2, &act, NULL) == -1))
    perror("Unable to set up signal handler!!");

  int sleeptime = atoi(argv[1]);

  pid_t pid = fork();

  if(pid == -1){
    perror("Failed to fork!!");
    return 1;
  } else if (pid == 0) {
    cpid = fork();
    if(cpid == -1) {
      perror("Failed on second fork!!");
      return 1;
    }else if(cpid == 0) {
      execvp(argv[2], &argv[2]);
      perror("execvp failed in child process!!");
    }else {
      int retval = wait(NULL);
      kill(getppid(), SIGUSR1);
      exit(0);
    return 1;
    }
  }
  else {
    sleep(sleeptime);
    kill(pid, SIGUSR2);
    wait(NULL);
    exit(1);
  }


  return 0;

}
